==> pan-0.6.4.tar.gz: pan-0.6.4/pan/group.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include "pan.h"

/*
 * globals 
 */

int group_mode = GROUP_ALL;
GtkWidget *groups_clist_menu;

/* 
 * callback function 
 */
void
click_groups_column_cb (GtkCList *clist, int n)
{
	if (n)
		return;
	switch (group_mode) {
	case GROUP_SUBSCRIBED:
		group_mode = GROUP_NEW;
		break;
	case GROUP_NEW:
		group_mode = GROUP_ALL;
		break;
	case GROUP_ALL:
		group_mode = GROUP_SUBSCRIBED;
		break;
	}
	grouplist_update ();
}

void
group_set_attrib_string (const group_data *gdata, const char *attrib, const char *value)
{
	if ( gdata->name )
	{
		void* db = server_get_attrib_db ( );
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		pan_db_put_value_str ( db, key, value );
		pan_db_sync ( db );
		g_free ( key );
	}
}


gchar*
group_get_attrib_string (const group_data *gdata, const char *attrib)
{
	if ( !gdata->name )
		return NULL;
	else {
		void* db = server_get_attrib_db ( );
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		gchar* retval = pan_db_get_value_str ( db, key );
		g_free ( key );
		return retval;
	}
}


/*--------------------------------------------------------------------
 * easily set the ~/.gnome/Pan sections for [alt.group.name]
 *--------------------------------------------------------------------*/

void
group_set_attrib_i (const group_data *gdata, const char *attrib, int value)
{
	if ( gdata && gdata->name && attrib )
	{
		void* db = server_get_attrib_db ( );
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		pan_db_put_value_i ( db, key, value );
		pan_db_sync ( db );
		g_free ( key );

		/* FIXME this isn't broken, but it's kind of a kludge */
		if ( !strcmp(attrib,"Total") || !strcmp(attrib,"Read") )
			grouplist_update_row ( gdata );
	}
}


int
group_get_attrib_i (const group_data *gdata, const char *attrib)
{

	if ( gdata && gdata->name && attrib )
	{
		void* db = server_get_attrib_db ( );
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		int retval = pan_db_get_value_i ( db, key );
		g_free ( key );
		return retval;
	}
	else return 0;
}


void
group_add_attrib_i (const group_data *gdata, const char *attrib, int i)
{
	if ( gdata && gdata->name && attrib && i )
	{
		void* db = server_get_attrib_db ( );
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		pan_db_add_value_i ( db, key, i );
		pan_db_sync ( db );
		g_free ( key );

		/* Make the change visible */
		grouplist_update_row (gdata);
	}
}


gint
compare_group_data (group_data *a, group_data *b)
{
	return strcmp(a->name, b->name);
}


void
check_new_group_list_for_newness ( GSList* list )
{
	/* assumes list and Pan.grouplist are sorted by name.. */
	GSList* a = list;
	GSList* b = Pan.grouplist;
	while ( a!=NULL && b!=NULL )
	{
		group_data* group_a = (group_data*) a->data;
		const group_data* group_b = (group_data*) b->data;
		const int rel = strcmp ( group_a->name, group_b->name );

		if ( rel>0 ) { /* old group not in new list */
		   b = b->next;
		   continue;
		}

		/* group_a is new if it's actually new or just
		   matches an old group still flagged as new... */
		if ( rel!=0 || (group_b->flags&GROUP_NEW) )
			group_a->flags |= GROUP_NEW;
		else
			group_a->flags &= ~GROUP_NEW;

		a = a->next;

		if ( !rel )
			b = b->next;
	}
	while ( a!=NULL ) /* exhausted Pan.grouplist, so all these are new... */
	{
	   ((group_data*)a->data)->flags |= GROUP_NEW;
	   a = a->next;
	}
}


void
add_group_to_clist (group_data *gdata)
{
	int row;
	char *text[4];
	GtkCListRow *clistrow;
	char unread_str[8] = { '\0' };
	char total_str[8] = { '\0' };

	const int total = group_get_attrib_i ( gdata, "Total" );
	if ( total )
	{
		int unread = total - group_get_attrib_i ( gdata, "Read" );
		sprintf ( total_str, "%d", total );
		sprintf ( unread_str, "%d", unread );
	}
	text[0] = gdata->name;
	text[1] = unread_str;
	text[2] = total_str;
	text[3] = NULL;

	row = gtk_clist_append (GTK_CLIST(Pan.group_clist), text);

	if ( total ) {
		gtk_clist_set_pixtext (GTK_CLIST(Pan.group_clist), row, 0, gdata->name, 5,
				       disk_pixmap->pixmap, disk_pixmap->mask);
	}

	/* The following hack is *MUCH* faster than this:
	 * gtk_clist_set_row_data(GTK_CLIST(Pan.group_clist), row, gdata);
	 * Let's hope the gtk structure doesn't change or something
	 */
	clistrow = (GtkCListRow *) GTK_CLIST(Pan.group_clist)->row_list_end->data;
	clistrow->data = gdata;
}


void
add_subscribed_group_to_clist (group_data *gdata)
{
	if (gdata->flags & GROUP_SUBSCRIBED)
		add_group_to_clist (gdata);
}


void
add_new_group_to_clist (group_data *gdata)
{
	if (gdata->flags & GROUP_NEW)
		add_group_to_clist (gdata);
}


void
group_dialog_clicked_cb (GnomeDialog* dialog, int num)
{
	switch (num) {
	case 0:					/* OK button */
		queue_add_grouplist_download ();
	default:
		gtk_widget_destroy (GTK_WIDGET(dialog));
		break;
	}
}


void
group_free (group_data *gdata)
{
	g_free (gdata);
}


void
get_all_groups (void)
{
	queue_add_grouplist_download ();
}


void
choose_download_dialog_clicked_cb (GnomeDialog *dialog,
				   int button_number, gpointer gdata)
{
	switch (button_number) {
	case 0:					/* All Headers */
		queue_add_articlelist_download (gdata,
						QUEUE_ARTICLELIST_DOWNLOAD_ALL |
						QUEUE_FOREGROUND);
		break;
	case 1:					/* New Headers */
		queue_add_articlelist_download (gdata,
						QUEUE_ARTICLELIST_DOWNLOAD_SAMPLE |
						QUEUE_FOREGROUND);
		break;
	default:
		/* Cancel */
		break;
	};
	gtk_widget_destroy (GTK_WIDGET(dialog));
}


void
choose_download_dialog (group_data *gdata)
{
	GtkWidget *dialog;
	GtkWidget *label;
	char *temp;

	dialog = gnome_dialog_new (_("Select Header Download Option"),
				   _("Download All Headers"),
				   _("Sample Some Headers"),
				   GNOME_STOCK_BUTTON_CANCEL, NULL);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (choose_download_dialog_clicked_cb),
			    gdata);

	temp = g_strdup_printf (_("Select an option for group: %s"), gdata->name);
	label = gtk_label_new (temp);
	g_free (temp);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);

	gui_popup_draw (dialog);
}


void
group_unload (group_data *gdata)
{
	if ( gdata == Pan.active_group )
		group_set_active_group ( NULL );
}

static article_db active_group_header_db = NULL;

void
group_set_active_group (
	group_data* g )
{
	if ( g == Pan.active_group )
		return;

	/* OUT WITH THE OLD GROUP */

	if ( Pan.active_group != NULL )
	{
		ahdb_unref ( active_group_header_db );
		active_group_header_db = NULL;
	}

	/* BRING IN THE NEW GROUP */

	Pan.active_group = g;
	if ( g != NULL )
	{
		/* update the title */
		gui_set_title ( g );

		/* open the article header database */
		active_group_header_db = ahdb_ref ( Pan.active_server, g );

		/* get the latest headers... */
		if ( ahdb_exists ( Pan.active_server, g ) )
			queue_add_articlelist_download ( g, QUEUE_ARTICLELIST_LOAD|QUEUE_FOREGROUND );
		else
			choose_download_dialog ( g );

	}
	else
	{
		articlelist_clear ( );
		gui_set_title ( NULL );
	}

}


/*---[ group_empty ]--------------------------------------------------
 * actual function that empties the group and deletes the groups
 * database file and associated gnome_config sections
 *--------------------------------------------------------------------*/
void
group_empty (group_data *gdata)
{
	if ( gdata )
	{
		gchar* section = g_strdup_printf ( "/Pan/%s:%s", Pan.active_server->name, gdata->name );

		group_unload (gdata);

		if (gnome_config_has_section (section)) {
			gnome_config_clean_section (section);
			gnome_config_sync();
		}

		g_free ( section );

		/* clear out the article headers */
		if ( gdata == Pan.active_group )
			ahdb_erase_all ( active_group_header_db );
		else
			ahdb_delete_db_file ( Pan.active_server, gdata );

		/* flush the flags */
		/* if (gdata->flags & GROUP_SUBSCRIBED)
			gdata->flags &= ~GROUP_ONDISK;
		else 
			gdata->flags = 0;
		*/
	}
}


/*--------------------------------------------------------------------
 * callback for group_empty, called when a button in group_empty_dialog
 * is clicked
 *--------------------------------------------------------------------*/
void
group_empty_cb (GnomeDialog *dialog,
		int button_number, gpointer gdata)
{
	switch (button_number) {
	case 0:					/* 'yes' */
		group_empty (gdata);
		break;
	case 1:					/* 'no' */
		break;
	default:				/* cancel */
		break;
	};

//	if (group_mode == GROUP_SUBSCRIBED)
//		grouplist_update ();
//	else
//		grouplist_update_row (gdata);
	if (group_mode == GROUP_SUBSCRIBED)
		grouplist_update ();

	gtk_widget_destroy (GTK_WIDGET(dialog));
}


/*--------------------------------------------------------------------
 * dialog popup asking user if they'd like to remove all the messages
 * for a given group.  called when the user tries to unsubscribe
 * or they choose the menu option to empty the group manually
 *--------------------------------------------------------------------*/
void
group_empty_dialog (group_data *gdata)
{
	GtkWidget *dialog;
	GtkWidget *label;
	gchar *temp;

	pan_lock ();
//	if (group_get_attrib_i (gdata, "Total") <= 0) {
//		group_unload (gdata);
//		if (group_mode == GROUP_SUBSCRIBED)
//			grouplist_update();
//		pan_unlock();
//		return;
//	}

	dialog = gnome_dialog_new (_("Clean group"),
				   GNOME_STOCK_BUTTON_YES,
				   GNOME_STOCK_BUTTON_NO,
				   NULL);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (group_empty_cb),
			    gdata);

	temp = g_strdup_printf (_("Remove all messages from group: \n\n%s ?"), gdata->name);
	label = gtk_label_new (temp);
	g_free (temp);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);
	gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
	gui_popup_draw (dialog);
	pan_unlock ();
}

//============================================================================

//        if ( 0 < group_db_store_article_header_gslist ( gdata, list, 0 ) )
//		gdata->flags |= GROUP_ONDISK;
//        gui_set_progress ( 1 );

==> pan-0.7.0.tar.gz: pan-0.7.0/pan/group.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include "group.h"
#include "grouplist.h"
#include "pan.h"
#include "queue.h"
#include "server.h"
#include "queue-item-headers.h"
#include "queue-item-grouplist.h"

/***
 *** Global Variables
 ***/

/***
 *** Public Routines
 ***/



group_data *
group_copy (group_data *src)
{
	group_data *dest;

	dest = (group_data *) g_malloc0 (sizeof(group_data));

	dest->name = g_strdup (src->name);
	dest->description = g_strdup (src->description);
	dest->flags = src->flags;

	return dest;
}


void
group_set_attrib_string (const server_data *sdata, const group_data *gdata, const char *attrib, const char *value)
{
	if ( gdata->name )
	{
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		pan_db_put_value_str ( sdata->db, key, value );
		pan_db_sync ( sdata->db );
		g_free ( key );
	}
}


gchar*
group_get_attrib_string (const server_data *sdata, const group_data *gdata, const char *attrib)
{
	if ( !gdata->name )
		return NULL;
	else {
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		gchar* retval = pan_db_get_value_str ( sdata->db, key );
		g_free ( key );
		return retval;
	}
}


void
group_set_attrib_i (const server_data *sdata, const group_data *gdata, const char *attrib, int value)
{
	if ( gdata && gdata->name && attrib )
	{
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		pan_db_put_value_i ( sdata->db, key, value );
		pan_db_sync ( sdata->db );
		g_free ( key );

		/* FIXME this isn't broken, but it's kind of a kludge */
		if ( !strcmp(attrib,"Total") || !strcmp(attrib,"Read") )
			grouplist_update_row ( gdata );
	}
}


int
group_get_attrib_i (const server_data *sdata, const group_data *gdata, const char *attrib)
{

	if ( sdata && gdata && gdata->name && attrib )
	{
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		int retval = pan_db_get_value_i ( sdata->db, key );
		g_free ( key );
		return retval;
	}
	else return 0;
}


void
group_add_attrib_i (const server_data *sdata, const group_data *gdata, const char *attrib, int i)
{
	if ( sdata && gdata && gdata->name && attrib && i )
	{
		gchar* key = g_strdup_printf ( "%s.%s", gdata->name, attrib );
		pan_db_add_value_i ( sdata->db, key, i );
		pan_db_sync ( sdata->db );
		g_free ( key );

		/* Make the change visible */
		grouplist_update_row (gdata);
	}
}


int
compare_group_data_std (const void* va, const void* vb)
{
	return compare_group_data (*(group_data**)va, *(group_data**)vb);

}
gint
compare_group_data (group_data *a, group_data *b)
{
	return strcmp(a->name, b->name);
}


void
check_new_group_list_for_newness ( GSList* list )
{
	/* assumes list and Pan.grouplist are sorted by name.. */
	GSList* a = list;
	server_data *sdata = grouplist_get_current_server ();
	GSList* b = (sdata ? sdata->grouplist : NULL);
	while ( a!=NULL && b!=NULL )
	{
		group_data* group_a = (group_data*) a->data;
		const group_data* group_b = (group_data*) b->data;
		const int rel = strcmp ( group_a->name, group_b->name );

		if ( rel>0 ) { /* old group not in new list */
		   b = b->next;
		   continue;
		}

		/* group_a is new if it's actually new or just
		   matches an old group still flagged as new... */
		if ( rel!=0 || (group_b->flags&GROUP_NEW) )
			group_a->flags |= GROUP_NEW;
		else
			group_a->flags &= ~GROUP_NEW;

		a = a->next;

		if ( !rel )
			b = b->next;
	}
	while ( a!=NULL ) /* exhausted Pan.grouplist, so all these are new... */
	{
	   ((group_data*)a->data)->flags |= GROUP_NEW;
	   a = a->next;
	}
}

void
group_free (group_data *gdata)
{
	g_free (gdata->name);
	g_free (gdata->description);
	g_free (gdata);
}

static void
choose_download_dialog_clicked_cb (GnomeDialog *dialog,
				   int button_number, gpointer gdata)
{
	server_data* sdata =
		(server_data*)gtk_object_get_data(GTK_OBJECT(dialog),"server");
	g_assert (sdata!=NULL);

	switch (button_number)
	{
		case 0: /* All Headers */
		{
			queue_add(QUEUE_ITEM(
				queue_item_headers_new(
					sdata, TRUE, gdata, HEADERS_ALL)));
			break;
		}
		case 1: /* New Headers */
		{
			queue_add(QUEUE_ITEM(
				queue_item_headers_new(
					sdata, TRUE, gdata, HEADERS_SAMPLE)));
			break;
		}
		default: /* Cancel */
		{
			break;
		}
	};

	gtk_widget_destroy (GTK_WIDGET(dialog));
}

typedef struct
{
	const server_data *sdata;
	group_data *gdata;
}
DownloadDialogStruct;

static int
choose_download_dialog_mainthread (gpointer data)
{
	DownloadDialogStruct *dds = (DownloadDialogStruct*) data;
	const server_data *sdata = dds->sdata;
	group_data *gdata = dds->gdata;
	GtkWidget *dialog;
	GtkWidget *label;
	char *temp;

	dialog = gnome_dialog_new (_("Select Header Download Option"),
				   _("Download All Headers"),
				   _("Sample Some Headers"),
				   GNOME_STOCK_BUTTON_CANCEL, NULL);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (choose_download_dialog_clicked_cb),
			    gdata);
	gtk_object_set_data(GTK_OBJECT(dialog), "server", (gpointer)sdata);

	temp = g_strdup_printf (_("Select an option for group: %s"), gdata->name);
	label = gtk_label_new (temp);
	g_free (temp);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);

	gui_popup_draw (dialog);
	g_free (data);

	return 0;
}

void
choose_download_dialog (const server_data *sdata, group_data *gdata)
{
	DownloadDialogStruct *dds = g_new(DownloadDialogStruct, 1);
	dds->sdata = sdata;
	dds->gdata = gdata;
	gui_invoke_in_gui_thread (choose_download_dialog_mainthread, dds);
}


/*---[ group_empty ]--------------------------------------------------
 * actual function that empties the group and deletes the groups
 * database file and associated gnome_config sections
 *--------------------------------------------------------------------*/
void
group_empty (server_data *sdata, group_data *gdata)
{
	if ( gdata )
	{
		/* unload group (if it's loaded) */
		if (articlelist_get_current_group() == gdata) {
			articlelist_clear ();
		}

		/* clear out the article headers */
		ahdb_delete_db_file ( sdata, gdata );

		/* wipe out all records */
		group_set_attrib_i (sdata, gdata, "Total", 0);
		group_set_attrib_i (sdata, gdata, "Read", 0);

		/* flush the flags */
		if (gdata->flags & GROUP_SUBSCRIBED)
			gdata->flags = GROUP_SUBSCRIBED;
		else
			gdata->flags = 0;

		/* update display */
		grouplist_update_row ( gdata );
	}
}


/*--------------------------------------------------------------------
 * callback for group_empty, called when a button in group_empty_dialog
 * is clicked
 *--------------------------------------------------------------------*/
static
void
group_empty_cb (GnomeDialog *dialog,
		int button_number, gpointer gdata)
{
	switch (button_number) {
	case 0:					/* 'yes' */
		group_empty (grouplist_get_current_server (), gdata);
		break;
	case 1:					/* 'no' */
		break;
	default:				/* cancel */
		break;
	};


	gtk_widget_destroy (GTK_WIDGET(dialog));
}


/*--------------------------------------------------------------------
 * dialog popup asking user if they'd like to remove all the messages
 * for a given group.  called when the user tries to unsubscribe
 * or they choose the menu option to empty the group manually
 *--------------------------------------------------------------------*/

static gint
group_empty_dialog_mainthread (gpointer data)
{
	group_data *gdata = (group_data*) data;
	GtkWidget *dialog;
	GtkWidget *label;
	gchar *temp;

	dialog = gnome_dialog_new (_("Clean group"),
				   GNOME_STOCK_BUTTON_YES,
				   GNOME_STOCK_BUTTON_NO,
				   NULL);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (group_empty_cb),
			    gdata);

	temp = g_strdup_printf (_("Remove all messages from group: \n\n%s ?"), gdata->name);
	label = gtk_label_new (temp);
	g_free (temp);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);
	gui_popup_draw (dialog);

	return 0;
}
void
group_empty_dialog (group_data *gdata)
{
	gui_invoke_in_gui_thread (group_empty_dialog_mainthread, gdata);
}

//============================================================================

//        if ( 0 < group_db_store_article_header_gslist ( gdata, list, 0 ) )
//		gdata->flags |= GROUP_ONDISK;
//        gui_set_progress ( 1 );
